{% load i18n %} {% load l10n %} {% load rich_text %}

{{ information_text | rich_text }}
<div class="text-center">
  <p><strong>Número para pagamento: </strong>{{ msisdn }}</p>
</div>

{{ instructions | rich_text }}
