{% load i18n %}

<p>
  {% blocktrans trimmed %} After you submitted your order, we will redirect you
  to MPesa to complete your payment. You will then be redirected back here to
  get your tickets. {% endblocktrans %}
</p>
<dl class="dl-horizontal">
  <dt>{% trans "Payment method" %}</dt>
  <dd>{{ provider.public_name }}</dd>
</dl>
