{% load i18n %}

<dl class="dl-horizontal">
  <dt><strong>Resposta do M-Pesa</strong></dt>
  <dd>{{ description }}</dd>
  <dt><strong>ID da transação</strong></dt>
  <dd>{{ transaction }}</dd>
  <dt><strong>Código da resposta</strong></dt>
  <dd>{% firstof code "N/A" %}</dd>
</dl>
